#ifndef MMGEN_HPP
#define MMGEN_HPP

#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <vector>
#include <deque>
#include <algorithm>
#include <bitset>
#include<chrono>
#include<stdexcept>
#include<utility>
#include<tuple>
#include <functional>
#include <typeinfo>

//#define NDEBUG
#include<assert.h>

#include "RandomFuncs.h"


constexpr double dPI{3.141592653589793238463};
constexpr double minerr{1e-6};
constexpr int MinSampleSize=5;
constexpr int DefaultPopSize=500;
constexpr int DefaultSampleSize=100;
constexpr int continuous=150669;

constexpr double DEFAULT_FACTOR= 1.25;
//template<typename T=double>
//const T BIG=numeric_limits<T>::max();

template <typename Type>
inline
const Type BIG()
{
    return std::numeric_limits<Type>::max();
}

template <typename Type>
inline
const Type LOW()
{
    return std::numeric_limits<Type>::min();
}

const double qzero=nextafter(0.0,1.0); // non zero positive minimum value: next representable number after 0 in direction to 1

const double qbig = nextafter(BIG<double>(), 1.0); // next representable number after BIG in direction to 1, i.e. the greatest number lower than big
const double qlow = nextafter(LOW<double>(), 0.0); // next representable number after LOW in direction to 0, i.e. the lowest number higher than LOW

//const int bigint =nextafter(BIG<int>(),1);

//const doble MaxAge =qbig-1;

const int missing_value{-99};



using namespace std;


const int GenomeSize = 11; // the most significant bit will allow rec evolution

const int Extinction = 10;
const double raizd2 = sqrt(2.0);

const int MAXISOLATION=1;
const int numfuns=6;
const int numDcases=8; // 0.1 0.2 0.5 1 2 5 10 50

const bool fixseed=false;

double fool;

// ******* UTILITIES *******


template<typename T>
inline std::string NtoS(const T& number){
	ostringstream snumber;
	snumber << number;
	return snumber.str();
}

template<typename T>
inline string NtoS(const T& number, const size_t precision){
	ostringstream snumber;
	snumber<<setprecision(precision);
	snumber << number;
	return snumber.str();
}

//out<<fixed<<setprecision(1);


template<typename T=int>
void errormsg(const T& val,const string& name, const string& msg,bool append,bool finish)
{
/*

*/
    ofstream fout;
    if(append) fout.open(name, ofstream::app);
    else  fout.open(name, ofstream::out);
	fout<< msg<<endl;
	if(finish){
        exit((int)val);
	}

}

template <typename T=int>
inline void trim_left(string& s, const string& delimiters =" \f\n\r\t\v"){

    s.begin(),s.erase(0, s.find_first_not_of(delimiters) ) ;

}

/// from stackoverflow: https://stackoverflow.com/questions/216823/whats-the-best-way-to-trim-stdstring?utm_medium=organic&utm_source=google_rich_qa&utm_campaign=google_rich_qa
// trim from start (in place)
static inline void ltrim(std::string &s) {
    s.erase(s.begin(), std::find_if(s.begin(), s.end(), [](int ch) {
        return !std::isspace(ch);
    }));
}

// trim from end (in place)
static inline void rtrim(std::string &s) {
    s.erase(std::find_if(s.rbegin(), s.rend(), [](int ch) {
        return !std::isspace(ch);
    }).base(), s.end());
}

// trim from both ends (in place)
static inline void trim(std::string &s) {
    ltrim(s);
    rtrim(s);
}

// get the words separated by space or comma or semicolon
vector<string> get_words(const string& line)
{
	vector<string> words;

	string first_w ("");

	for(size_t i = 0; i < line.length(); i++)
	{
		if( line[i] !=' ' && line[i] !='\t' && line[i] !='\n' && line[i] !='\r' && line[i] !=',' && line[i] !=';')
		{
			first_w += line.substr(i, 1);

			if( line[i+1] ==' ' || line[i+1] =='\t' || line[i+1] =='\n'|| line[i+1] =='\r'
					|| line[i+1] =='\0' || line[i] ==',' || line[i] ==';')
			{
				words.push_back(first_w);

				first_w = "";
			}

		}
	}

	return words;
}

template <typename rT=double, typename iT=int>
void ReadUserPhenotypes(const string& path,const string& ufilename, const int& n1, const int& n2, vector<rT>& fems,rT& minfem, rT& maxfem, vector<rT>& males,rT& minmale, rT& maxmale){

//	datafile >> boolalpha;
//	cout<<boolalpha;
try{
    string temp;

    //if(!p1.empty()) p1.clear();
    //if(!p2.empty()) p2.clear();

//    assert (fems.size()==n1);
//    assert (males.size()==k2);

    string filename=path+ufilename;

    temp="------------------------------------------------\n";

    ifstream datafile; //ifstream will be opened for reading
    datafile.clear();
    datafile.open(filename.c_str());
    if(datafile.fail()){
        throw("Could not open file: "+filename);
    }

    temp= "\nReading file "+ filename;
    cout<<temp<<endl;

    vector<string> words;
    int n = n1+n2;
    int line=0, m=0;

    /// Read the file the format should be n1 female values + n2 male values e.g. if n=10 and n1=n2=5:
    /*
// these are read as females
0.1
0.2
0.01
0.0067
0.32
// these are read as males
0.407
0.3
0.11
0.0987
0.232
    */

    while(!datafile.eof()){

        datafile.clear();
        temp.clear();
        getline(datafile, temp);

        trim(temp); // delete spaces at the beginning left

        if(temp.empty()) continue; // skip empty lines

        //transform(temp.begin(),temp.end(), temp.begin(), (int(*)(int)) tolower); // transform to lower case

        if(temp.find("#")!=string::npos){ // each time it is found a # skip

            continue;

        } // close if find #
        else{ // manage info
            words = get_words(temp);

            if(words.size()>1){
                string msg = "Wrong format in user phenotypes file "+ufilename+" line "+NtoS(line+1) +"\n";
                cout<<msg<<endl;
                errormsg(words.size(),"MateSim.err", msg,true,true);
            }

            double a = atof(words[0].c_str());

            if (line<n1){

                fems[line]=a;
                if(minfem> fems[line]) minfem = fems[line];
                if(maxfem<fems[line]) maxfem = fems[line];
            }
            else if(line<n){
                m = line-n1;
                males[m]=a;
                if(minmale> males[m]) minmale = males[m];
                if(maxmale<males[m]) maxmale = males[m];
            }

            ++line;

        } // close else manage info
    } // close while

    //assert(line==n);
    if(line!=n){
        string msg = "Wrong number of phenotypes in user phenotypes file "+ufilename+": The number of phenotypes in the file ("+NtoS(line)+") does not coincide with the population size ("+NtoS(n) +"). Check your argument -N (by default set to "+NtoS(DefaultPopSize)+").\n";
        cout<<msg<<endl;
        errormsg(words.size(),"MateSim.err", msg,true,true);
    }

//    if(accumulate(fems.begin(),fems.end(),0.0)!=1.0){
//        string msg = "Female frequencies do not sum up to 1"+ufilename+". Check the minimum frequency is "+NtoS(MAF)+"\n";
//        cout<<msg<<endl;
//        errormsg(words.size(),"MateSim.err", msg,true,true);
//    }

    datafile.close();

} // close try
catch(const std::invalid_argument& e ){

    throw invalid_argument(e.what());
      //texterr(1,msg,true);
}
catch (std::exception& e){
    throw exception();
}

}


// Read user-defined population female and male frequency of types from file
template <typename rT=double, typename iT=int>
void ReadUserFreqs(const string& path,const string& ufilename, const int& k1, const int& k2, vector<rT>& p1, vector<rT>& p2, const double& MAF){

//	datafile >> boolalpha;
//	cout<<boolalpha;
try{
    string temp;

    //if(!p1.empty()) p1.clear();
    //if(!p2.empty()) p2.clear();

    assert (p1.size()==(unsigned)k1);
    assert (p2.size()==(unsigned)k2);

    string filename=path+ufilename;


    temp="------------------------------------------------\n";

    ifstream datafile; //ifstream will be opened for reading
    datafile.clear();
    datafile.open(filename.c_str());
    if(datafile.fail()){
        throw("Could not open file: "+filename);
    }

    temp= "\nReading file "+ filename;
    cout<<temp<<endl;

    vector<string> words;

    int line=0;

    /// Read the file the format shoudl be:
    /*
    #females freqs (k1=3)
    0.4 0.1 0.5

    #males freqs (k2=3)
    0.3 0.4    0.3
    */

    while(!datafile.eof()){

        datafile.clear();
        temp.clear();
        getline(datafile, temp);

        trim_left(temp); // delete spaces at the beginning left

        if(temp.empty()) continue; // skip empty lines

        //transform(temp.begin(),temp.end(), temp.begin(), (int(*)(int)) tolower); // transform to lower case

        if(temp.find("#")!=string::npos){ // each time it is found a # skip

            continue;

        } // close if find #
        else{ // manage info
            words = get_words(temp);
            int wordsSize= words.size();
            ++line;

            if(line==1){
                if(k1!=wordsSize){
                    string msg = "Wrong number of female types in user file "+ufilename+"\n";
                    cout<<msg<<endl;
                    errormsg(words.size(),"MateSim.err", msg,true,true);

                }

                if(k1>1) /// Dec 2019
                    for(int t=0; t<k1; ++t ){
                        double a = atof(words[t].c_str());
                        p1[t]=min(max(a,MAF),1.0-MAF);

                    }
                else /// k1=1 Dec 2019
                    p1[0]=1;
            }
            else if (line==2){
                if(k2!=wordsSize){
                    string msg = "Wrong number of male types in user file "+ufilename+"\n";
                    cout<<msg<<endl;
                    errormsg(words.size(),"MateSim.err", msg,true,true);
                }
                if(k2>1) /// Dec 2019
                    for(int t=0; t<k2; ++t ){
                        double a = atof(words[t].c_str());
                        p2[t]=min(max(a,MAF),1.0-MAF);
                    }
                else /// k2=1 Dec 2019
                    p2[0]=1;
            }

        } // close else manage info
    } // close while

    /// Dic 2019: improve check for female frequ
    double ff = accumulate(p1.begin(),p1.end(),0.0);
    if(fabs(ff-1.0)> minerr){
        p1.back() = 1.0 - accumulate(p1.begin(),p1.end()-1,0.0);
        string msg = "Warning: Female frequencies do not sum up to 1 in "+ufilename+". The last frequency has been updated to "+NtoS(p1.back())+"\n";
        cout<<msg<<endl;
        errormsg(words.size(),"MateSim.err", msg,true,false);
    }
/// Dic 2019: add check for male frequ
    double mf = accumulate(p2.begin(),p2.end(),0.0);
    if(fabs(mf-1.0)> minerr){
        p2.back() = 1.0 - accumulate(p2.begin(),p2.end()-1,0.0);
        string msg = "Warning: Male frequencies do not sum up to 1 in "+ufilename+". The last frequency has been updated to "+NtoS(p2.back())+"\n";
        cout<<msg<<endl;
        errormsg(words.size(),"MateSim.err", msg,true,false);
    }

    datafile.close();

} // close try
catch(const std::invalid_argument& e ){

    throw invalid_argument(e.what());
      //texterr(1,msg,true);
}
catch (std::exception& e){
    throw exception();
}

}


// Read user-defined individual preference values from file.
// This is only for generating the mating tables under specific encounter process so the consideration of the number of parameters is not important here because we do not perform estimates

// usage: ReadUserModels(path,userfile, positions,models_a_estims,uk1,uk2);
template <typename rT=double, typename iT=int>
void ReadUserModels(const string& path,const string& ufilename,  vector<vector<rT>>& models_a_estims, iT& uk1, iT& uk2){

//	datafile >> boolalpha;
//	cout<<boolalpha;
try{
    string temp;

    string filename=path+ufilename;


    temp="------------------------------------------------\n";

    ifstream datafile; //ifstream will be opened for reading
    datafile.clear();
    datafile.open(filename.c_str());
    if(datafile.fail()){
        throw("Could not open file: "+filename);
    }

    temp= "\nReading file "+ filename;
    cout<<temp<<endl;


    vector<string> words;

    int line=0;
    int numfem=0;
    while(!datafile.eof()){

        datafile.clear();
        temp.clear();
        getline(datafile, temp);

        trim_left(temp); // delete spaces at the beginning left

        if(temp.empty()) continue; // skip empty lines

        //transform(temp.begin(),temp.end(), temp.begin(), (int(*)(int)) tolower); // transform to lower case

        if(temp.find("#")!=string::npos){ // each time it is found a # skip

            continue;

        } // close if find #
        else{ // manage info
            words = get_words(temp);
            ++line;

            if(line==1){ // num of female and male types

                assert(words.size()>1);
                assert(words.size()<3);

                int k1=atoi(words[0].c_str());
                int k2=atoi(words[1].c_str());

                if(k1!=uk1 || k2!=uk2){ // the number of types predefined is different of the number of types in the file
                    models_a_estims.clear();
                    models_a_estims.resize(k1,vector<rT>(k2));
                    uk1=k1;
                    uk2=k2;
                }
            }
            else{ // read for each female line k2 male types
                assert((unsigned)uk2==words.size());
                for(int t=0; t<uk2; ++t ){
                    // check acceptable ranges of parameters
                    double a = atof(words[t].c_str());
                    if(a<=0){
                        string msg("Negative parameter " + NtoS(a)+ " in user model" +"\n");
                        cout<<msg<<endl;
                        errormsg(a,"MateSim_ERROR.log",msg,false,true);
                    }

                    models_a_estims[numfem][t] =a;
                }
                ++numfem;
                if(numfem==uk1){ // we are done
//                    assert(models_a_estims.size()==(uk1));
//                    for(int f=0;f<uk1;++f)
//                        assert(models_a_estims[f].size()==(uk2));
                    break;
                }
            }// close else reading each female line

        } // close else manage info
    } // close while

    datafile.close();

} // close try
catch(const std::invalid_argument& e ){

    throw invalid_argument(e.what());
      //texterr(1,msg,true);
}
catch (std::exception& e){
    throw exception();
}

}


/// Perform a roulette wheel selection of one individual or event from k classes
/// and selected proportional to the corresponding probability p of each class. The  selected individual is stored in the corresponding index in obs.
/// the  selected index is returned
/// if append is true the numbers in obs can accumulate over different calls of the function
template<typename iT=int, typename rT=double, typename rTcol=vector<rT> >
iT roulette(const rTcol& p, const bool append, rTcol& obs ){

    int acum=0;
    iT index =-1;
    iT k= obs.size();
    if(!append) std::fill(obs.begin(), obs.end(), 0.0);

    // roulette
    double acumfreq=0.0;
    double u=uniform(randgen);
    double prev=0;
    for(int f=0; f< k; ++f ){
        acumfreq+=p[f];
        if( (u>=prev) && (u<acumfreq) ){
            ++obs[f];
            ++acum;
            index=f;
            break;
        }
        else{
            prev=acumfreq;
        }

    }


    return index;
}
/// This roulete is specialized for mass EM model, it also stores the indexes of the chosen individuals
/// it selects up to n individuals however the availability of the individuals must be checked i.e. it could be the
// we have 5 indivs type 1 and 5 type 2 (p=0.5) but if n=10 it may happen that by random we get 6 4 and we have not 6  but 5
template<typename iT=int, typename rT=double, typename iTcol=vector<iT> , typename rTcol=vector<rT>>
void roulette(const iT& n,const rTcol& p, rTcol& remain, iTcol& indexes){

assert((unsigned)n==indexes.size());
    int acum=0;
    iT k= remain.size();

    std::fill(indexes.begin(), indexes.end(), -1);

    do{ // obtain n1 females distributed in k1 types. The highest n1 the more resemble the observed to the user defined frequencies
        // roulette
        double acumfreq=0.0;
        double u=uniform(randgen);
        double prev=0;
        for(int f=0; f< k; ++f ){
            acumfreq+=p[f];
            if( (u>=prev) && (u<acumfreq) ){
                if(remain[f]>0){
                    --remain[f];
                    indexes[acum]=f;
                    ++acum;
                    break;
                }
                else{
//cout<<p[f]<<"\t"<<acumfreq<<endl;
                    if(acumfreq<1){
                        prev=acumfreq;
                        continue;
                    }
                    else{
                        break;
                    }

                }
            }
            else{
                prev=acumfreq;
            }

        }
    }while(acum<n);

}

/// Write the table
template<typename T=double>
void writediscretemodel(const string& ofilename,const string& programname,const string& comments,const int& format, const vector<T>& p1, const vector<T>& p2, const vector<vector<T>>& matings){

    ofstream out(ofilename.c_str(),ios::out);
    // Set the current local date
    time_t rawtime;
    struct tm * timeinfo;

    time ( &rawtime );
    timeinfo = localtime ( &rawtime );

    out<<"##"<<asctime(timeinfo)<<endl;
    out<<"##------------------------------------------------\n##";

    string mensaje= programname +"\n";

    mensaje+=
	"##---------------------------------------------------------------------\n";
	mensaje+="##\n";
	mensaje+="##by A Carvajal-Rodriguez			\n";
	mensaje+="##http://acraaj.webs.uvigo.es				\n";
	mensaje+="##---------------------------------------------------------------------\n\n##";

    mensaje+=comments+"\n";
	out<<mensaje;

// paint the table

    out<<"# format number\n";
    out<<format<<endl;
    out<<"# num of types\n";
    if(format==0)
        out<< p1.size()<<endl;
    else
        if(format==2 || format==15)
            out<< p1.size()<<"\t"<<p2.size()<<endl;
out<<fixed<<setprecision(1);
    out<<"#premating male numbers\n";
    for(auto male:p2)
        out<<male<<"\t";
    out<<endl;

    out<<"#premating female numbers\n";
    for(auto fem:p1)
        out<<fem<<"\t";
    out<<endl;

    out<<"# matings by rows (females)\n";

    out<<fixed<<setprecision(1);

    int k1=matings.size();
    int k2= matings[0].size();
    for(int r=0; r< k1; ++r){
        for(int c=0;c<k2;++c){
            out<<matings[r][c]<<"\t";
        }
        out<<"\n";
    }


    out.close();

}

/// write continuous phenotypes
template<typename T=double>
void writephenotypedistrib(const string& ofilename,const string& programname,const int& n1, const int& n2, const vector<T>& p1, const vector<T>& p2){


assert((unsigned)n1==p1.size());
assert((unsigned)n2==p2.size());


    ofstream out(ofilename.c_str(),ios::out);
    // Set the current local date
    time_t rawtime;
    struct tm * timeinfo;

    time ( &rawtime );
    timeinfo = localtime ( &rawtime );

    out<<asctime(timeinfo)<<endl;
    out<<"------------------------------------------------\n";

    string mensaje= programname +"\n";

    mensaje+=
	"---------------------------------------------------------------------\n";
	mensaje+="\n";
	mensaje+="by A Carvajal-Rodriguez			\n";
	mensaje+="http://acraaj.webs.uvigo.es				\n";
	mensaje+="---------------------------------------------------------------------\n\n";

    mensaje+="Phenotypic distribution: ";
    mensaje+="\n";
//	out<<mensaje;
//
//    out<<"Number of females: ";
//    out<<n1<<". ";
//
//    out<<"Number of males: ";
//    out<<n2<<endl;
//
//    out<<"Female\tMale\n";

out<<"\n#females\n";

    for(int f=0; f<n1; ++f){
        out<<p1[f]<<endl;
    }

out<<"\n#males\n";

    for(int m=0; m<n2; ++m){
        out<<p2[m]<<endl;
    }

    out<<endl;
    out.close();

}

/// Write the table
template<typename T=double>
void writecontinuousmodel(const string& ofilename,const string& programname,const string& comments,const bool EM, const bool massEM,const int& n1, const int& n2, const vector<T>& p1, const vector<T>& p2, const int& samplesize){

    ofstream out(ofilename.c_str(),ios::out);
    // Set the current local date
    time_t rawtime;
    struct tm * timeinfo;

    time ( &rawtime );
    timeinfo = localtime ( &rawtime );

    out<<asctime(timeinfo)<<endl;
    out<<"------------------------------------------------\n";

    string mensaje= programname +"\n";

    mensaje+=
	"---------------------------------------------------------------------\n";
	mensaje+="\n";
	mensaje+="by A Carvajal-Rodriguez			\n";
	mensaje+="http://acraaj.webs.uvigo.es				\n";
	mensaje+="---------------------------------------------------------------------\n\n";

    mensaje+="Preference function: ";
    mensaje+=comments+"\n";
	out<<mensaje;

// paint the table with female and male columns


    string matingmodel = "\nEncounter model: ";
    if(EM)
        if(massEM)
            matingmodel+="Mass EM\n";
        else
            matingmodel+="Indiv EM\n";
    else
        matingmodel+="With replacement EM\n";

    out<<matingmodel<<endl;

    out<<"Premating female numbers: ";
    out<<n1<<". ";

    out<<"Premating male numbers: ";
    out<<n2<<endl;
    out<<"Sample size "<<samplesize<<endl;
    out<<"Matings\n";
    out<<"Female\tMale\n";
//out<<fixed<<setprecision(1);

    int expectedsamplesize= p1.size();
    int missed=0;
    for(int f=0; f<expectedsamplesize; ++f){
        if(p1[f]!= missing_value && p2[f]!= missing_value)
            out<<p1[f]<<"\t"<<p2[f]<<endl;
        else
            ++missed;
    }

    assert(samplesize == (expectedsamplesize-missed));

    out<<endl;

    out.close();
}



/// Write the correlation obtained for each sample of matings
template<typename T=double>
void writecorrelations(const string& ofilename,const string& programname,const string& comments,const bool EM, const bool massEM,const int& n1, const int& n2,const int& samplesize, const vector<string>& filenames, const vector<T>& rho, const T& femage, const T&maleage){

    ofstream out(ofilename.c_str(),ios::out);
    // Set the current local date
    time_t rawtime;
    struct tm * timeinfo;

    time ( &rawtime );
    timeinfo = localtime ( &rawtime );

    out<<asctime(timeinfo)<<endl;
    out<<"------------------------------------------------\n";

    string mensaje= programname +"\n";

    mensaje+=
	"---------------------------------------------------------------------\n";
	mensaje+="\n";
	mensaje+="by A Carvajal-Rodriguez			\n";
	mensaje+="http://acraaj.webs.uvigo.es				\n";
	mensaje+="---------------------------------------------------------------------\n\n";

    mensaje+="Preference function: ";
    mensaje+=comments+"\n";
	out<<mensaje;

    string matingmodel = "\nEncounter model: ";
    if(EM)
        if(massEM)
            matingmodel+="Mass EM\n";
        else
            matingmodel+="Indiv EM\n";
    else
        matingmodel+="With replacement EM\n";

    out<<matingmodel<<endl;

    out<<"Premating female numbers: ";
    out<<n1<<". ";

    out<<"Premating male numbers: ";
    out<<n2<<"."<<endl;

    if(femage>0 && maleage>0){
        out<<"Mean female age after the mating season: ";
        out<<femage<<endl;
        out<<"Mean male age after the mating season: ";
        out<<maleage<<endl;
    }


    out<<"Mean sample size:\t";
    out<<samplesize<<endl;

    int numfiles = rho.size();
    out<<"File\tRho\n";
    for(int f=0; f<numfiles; ++f)
        if(rho[f]!= missing_value)
            out<<filenames[f]<<"\t"<<rho[f]<<endl;

    out<<endl;

    out.close();
}


/**

Computes a preference value based on the preference function pref_func
For continuous traits a generalization of the preference function is

f = exp(alpha0 + alpha1*Z)

Therefore three first arguments of pref_func are the first three positions of the vector Tvec containing alpha0, alpha1 and Z
remaining arguments can be used for parameters related with the previous ones e.g. if args[1] =alpha1 = C^2/s^2; args[3] could be = C so that be know if is a positive assortative or negative assortative case

**/


template<typename T=double,typename Tvec=vector<T>>
T computepropensity(Tvec& args, T (&pref_func)(Tvec&)){

    return pref_func(args);
}

/// The difference between the different  versions in table 1 of CR & RA 2014 concerns to the values stored in args[1] and args[2]
template<typename T=double,typename Tvec=vector<T>>
T Gaussbasedpref(Tvec& args){

    if(args.size()< 4){
        string msg("Gaussbasedpref: Bad argument number: " + NtoS(args.size())+ " in Gaussbasedpref" +"\n");
        cout<<msg<<endl;
        errormsg(args.size(),"MateSim_ERROR.log",msg,false,true);
    }
    else if(args[1]>=0){ //because alpha1 in the exponent must be negative
            string msg("Gaussbasedpref: Bad argument: " + NtoS(args[1])+ " in args[1]" +"\n");
            cout<<msg<<endl;
            errormsg(args[1],"MateSim_ERROR.log",msg,false,true);
    }

    double choice = args[3];


    if(choice==0)
        return 1;

    return exp(args[1]*args[2]);

}
/// Probabilities based in Xie et al 2015.
/// Args[0] to [2] corresponds to the female will for male j and args[3] to [5] to the male will for female i
template<typename T=double,typename Tvec=vector<T>>
T Logitpref(Tvec& args){

    if(args.size()< 6){
        string msg("Logitpref: Bad argument number: " + NtoS(args.size())+ " in Logitpref" +"\n");
        cout<<msg<<endl;
        errormsg(args.size(),"MateSim_ERROR.log",msg,false,true);
    }
    double fi = exp(args[0] + args[1]*args[2]);
    fi/= (1.0+fi);

    double fj = exp(args[3] + args[4]*args[5]);
    fj/= (1.0+fj);

    return fi*fj;

}

/***********************************************************************************
AUTHOR: A Carvajal-Rodriguez
DATE:	081016
FUNCTION: regcorrelxy(container<T> x, container<T>y)
INPUT: two STL containers of size n
PERFORM: computes unbiased (or not) estimation of correlation following biometry 15.3 p.559-560.
         Also performs linear or inverse regression estimating a and b which are returned in the pair<rT,rT>
OUTPUT: the standard or unbiased estimate rho of correlation coefficient
************************************************************************************/
template <typename rT=double,typename colT=double, typename Col=vector<colT>>
rT regcorrelxy(const colT& x0, const colT&y,const bool& unbiased,const bool& inverse, pair<rT,rT>& regression){

	if(x0.size()!=y.size()){
		throw invalid_argument("Sample size different in regcorrelxy");
	}
	if(x0.empty()){
        throw invalid_argument("Empty vector in regcorrelxy");
	}

	int n=x0.size();
    int nm_n{n}; // nonmissing_n
    colT x{x0};
    if(inverse){
        for(auto& el:x){
            el = (el>0)? (1.0/el): missing_value;
            if(el<=missing_value) throw invalid_argument("division by 0 in inverse regression\n");
        }
    }

//for(int ix=0; ix<n;++ix)
//    cout<< x0[ix]<<"\t"<<x[ix]<<endl;
//exit(0);

	rT sumx{0}, sumy{0},sumxy=0, sumx2=0,sumy2=0, avex,avey;
	rT rho,denom;
	rT x2desv, b,a;

	for(int i=0;i <n;++i){
		if(y[i]!=missing_value){
            sumx+=x[i];
            sumy+=y[i];
            sumxy+= x[i]*y[i];
            sumx2+=x[i]*x[i];
            sumy2+=y[i]*y[i];
		}
		else --nm_n;
	}

    if(nm_n<=0){
        throw invalid_argument("Too low sample size in regcorrelxy: "+NtoS(nm_n));
    }

	avex= sumx/nm_n;
	avey=sumy/nm_n;

	rho = sumxy - nm_n*avex*avey;

	x2desv = (sumx2- (sumx*sumx)/nm_n);

	// regression
	b = rho / x2desv ;
    a = avey - b*avex;
    regression = make_pair(a,b);

    rT y2desv = sumy2- (sumy*sumy)/nm_n;

	denom = x2desv*y2desv;

	if(denom<=0){
		//throw invalid_argument("Zero or Negative denominator in regcorrelxy: "+NtoS(x2desv)+" * "+ NtoS(y2desv));

		return 0;
    }

	denom=sqrt(denom);

    // correlation
	rho/=denom;

    if(unbiased){
        if(nm_n>4)
            rho=rho*(1 + (1-rho*rho)/(2*(nm_n-4)));
        else
            throw invalid_argument("Too low sample size in regcorrelxy: "+NtoS(nm_n));
    }

	return rho;
}


// Transpose in situ a matrix formed by any STL container of the form container<container<T>>
// the matrix passed as argument is transposed
/**
Usage:

vector<vector<double>>data(100,vector<double>(200000,0));
int id=1;
for(auto& d:data)
    for(auto& dd:d){
        dd+=id;
        ++id;
    }

cout<<data.size()<<" x " << data[0].size()<<endl;

//for(auto d:data)
//    printcol(d);

cout<<endl;
transposeinsitu(data);

cout<<data.size()<<" x " << data[0].size()<<endl;


**/
template <typename T= double, typename Tcol= vector<T>, typename Tcolofcol = vector<Tcol> >
inline void transposeinsitu(Tcolofcol& data) {

    Tcolofcol tdata;

    // This assumes that all inner vectors have the same size and
    // allocates space for the complete result in advance
    tdata.resize(data[0].size(), Tcol(data.size()));

    for (typename Tcol::size_type i = 0; i < data[0].size(); ++i)
        for (typename Tcol::size_type j = 0; j < data.size(); ++j) {
            tdata[i][j] = data[j][i];
        }
    data = tdata;

}



/*
*	Modified from NRC++ routines

*/


template <typename T= double>
T gammln(const T& xx){
	int j;
	T x,y,tmp,ser;
	static const double cof[6]={76.18009172947146,-86.50532032941677,24.01409824083091,
	-1.231739572450155,0.1208650973866179e-2,-0.5395239384953e-5};

	y=x=xx;
	tmp=x+5.5;
	tmp-=(x+0.5)*log(tmp);
	ser=1.000000000190015;
	for(j=0;j<6;++j) ser+=cof[j]/++y;
	return -tmp+log(2.5066282746310005*ser/x);
}

template <typename T= double>
void gcf(T& gammcf, const T& a, const T& x, T& gln){
	const int ITMAX=5000;
	const double EPS=numeric_limits<double>::epsilon();
	//const double EPS=numeric_limits<double>::epsilon() -> DBL_EPSILON;
	const double FPMIN=numeric_limits<double>::min()/EPS;
	int i;
	double an,b,c,d,del,h;

	gln=gammln(a);
	b=x+1.0-a;
	c=1.0/FPMIN;
	d=1.0/b;
	h=d;
	for(i=1;i<=ITMAX;++i){
		an= -i*(i-a);
		b+=2.0;
		d=an*d+b;
		if(fabs(d)<FPMIN) d=FPMIN;
		c=b+an/c;
		if(fabs(c)<FPMIN) c=FPMIN;
		d=1.0/d;
		del=d*c;
		h*=del;
		if(fabs(del-1.0) <= EPS) break;
	}

	if(i > ITMAX){
		string msg("a too large, ITMAX too small in routine gcf");
		cout<<msg<<" x = "<<x<<endl;
		errormsg(7,"MateSim_ERROR.log", msg,false,true);
	}
	gammcf=exp(-x+a*log(x)-gln)*h; // Put factors in front
}
template <typename T= double>
void gser(T& gamser, const T& a, const T& x, T& gln){
	const int ITMAX=5000;
	const double EPS= std::numeric_limits<double>::epsilon();
	//const double EPS=numeric_limits<double>::epsilon() -> DBL_EPSILON;
	int n;
	double sum,del,ap;

	gln=gammln(a);
	if(x<=0.0){
		if(x<0.0) errormsg(8,"MateSim_ERROR.log","x less than 0 in gser",false,true);
		gamser=0.0;
		return;
	}
	else{
		ap=a;
		del=sum=1.0/a;
		for(n=0;n<ITMAX;++n){
			++ap;
			del*=x/ap;
			sum+=del;
			if(fabs(del)<fabs(sum)*EPS){
				gamser=sum*exp(-x+a*log(x)-gln);
				return;
			}
		}
		string msg("a too large, ITMAX too small in routine gser");
		cout<<msg<<" x = "<<x<<endl;
		errormsg(9,"MateSim_ERROR.log",msg,false,true);

		return;
	}
}

template <typename T= double>
double gammp(const T& a, const T& x){
	double gamser,gammcf,gln;

	if(x<0.0||a<=0.0) errormsg(10,"MateSim_ERROR.log","Invalid arguments in gammp",false,true);
	if(x<a+1.0){ // use the series representation
		gser(gamser,a,x,gln);
		return gamser;
	}
	else{ // use the continued fraction representation
		gcf(gammcf,a,x,gln);
		return 1.0-gammcf; // and take its complement
	}
}
template <typename T= double>
double gammq(const T& a, const T& x){
	double gamser,gammcf,gln;
	if(x<0.0||a<=0.0){cout<<"x: "<<x<<" a: "<<a<<endl; errormsg(11,"MateSim_ERROR.log","Invalid arguments in gammq",false,true);}
	if(x<a+1.0){ // use the series representation
		gser(gamser,a,x,gln);
		return 1.0 - gamser;
	}
	else{ // use the continued fraction representation
		gcf(gammcf,a,x,gln);
		return gammcf; // and take its complement
	}
}
// Given a chissquare value and the degrees of freedom return the probability P
template <typename T= double>
double getChiPval(const T& chsk,const int & df){

	if(chsk==0.0) return 1.0;
	return gammq(0.5*df,0.5*chsk);
}

///write the matign rounds for each file
template<typename T=double>
void writeRoundsTable(const string& ofilename,const string& programname,const string& comments, const int& format, const vector<vector<double>>& p1_t, const vector<vector<double>>& p2_t, const vector<vector<vector<double>>>& matings, const vector<vector<double>>& qRprime, const vector<vector<double>>& qR,const vector<double>& JR,const vector<double>& pJR,const vector<double>& J,const vector<double>& pJ){

    ofstream out(ofilename.c_str(),ios::out);
    // Set the current local date
    time_t rawtime;
    struct tm * timeinfo;

    time ( &rawtime );
    timeinfo = localtime ( &rawtime );

    out<<"##"<<asctime(timeinfo)<<endl;
    out<<"##------------------------------------------------\n##";

    string mensaje= programname +"\n";

    mensaje+=
	"##---------------------------------------------------------------------\n";
	mensaje+="##\n";
	mensaje+="##by A Carvajal-Rodriguez			\n";
	mensaje+="##http://acraaj.webs.uvigo.es				\n";
	mensaje+="##---------------------------------------------------------------------\n\n##";

    mensaje+=comments+"\n";
	out<<mensaje;

// paint the table
    int R =p1_t.size();
    int k1 = p1_t[0].size();
    int k2 = p2_t[0].size();

    out<<"# format number\n";
    out<<format<<endl;
    out<<"# num of types\n";
    out<<k1<<"\t"<<k2<<endl;
out<<fixed<<setprecision(3);
    out<<"#Sampling over "+NtoS(R)+ " mating rounds\n";

    out<<endl;


string header ="Round\t";
for(int f=0; f< k1; ++f)
    header+="pfem_r"+NtoS(f+1)+"\t";
for(int m=0; m< k2; ++m)
    header+="pmale_r"+NtoS(m+1)+"\t";

for(int f=0; f< k1; ++f)
    for(int m=0; m< k2; ++m)
        header+=NtoS(f+1)+" x "+NtoS(m+1)+"\t";
header+="\n";
out<<header<<endl;

    vector<vector<int>>totals(k1,vector<int>(k2,0));
    for(int r=0; r<R;++r){
        out<<(r+1)<<"\t";
        for(int f=0; f< k1; ++f)
            out<<p1_t[r][f]<<"\t";
        for(int m=0; m< k2;++m)
            out<<p2_t[r][m]<<"\t";

        for(int f=0; f< k1; ++f){
            for(int m=0;m<k2;++m){
                out<<matings[r][f][m]<<"\t";
                totals[f][m]+=matings[r][f][m];
            }
        }
        out<<"\n";
    } // close for r

    out<<"Total\t";

    for(int f=0; f< k1-1; ++f)
        out<<"\t";
    out<<"\t";
    for(int m=0; m< k2-1;++m)
        out<<"\t";
    out<<"\t";

    for(int f=0; f< k1; ++f){
        for(int m=0;m<k2;++m){
            out<<totals[f][m]<<"\t";
        }
    }


    out<<"\n";

    out<<"\t"; // the Round column
    for(int f=0; f< k1-1; ++f) // p1 columns
        out<<"\t";
    out<<"\t";
    for(int m=0; m< k2-1;++m) // p2 columns
        out<<"\t";

    out<<"q'R\t";
    for(int f=0; f< k1; ++f){
        for(int m=0;m<k2;++m){
            out<<qRprime[f][m]<<"\t";
        }
    }

    out<<"\n";

    out<<"\t"; // the Round column
    for(int f=0; f< k1-1; ++f) // p1 columns
        out<<"\t";
    out<<"\t";
    for(int m=0; m< k2-1;++m) // p2 columns
        out<<"\t";

    out<<"qR\t";
    for(int f=0; f< k1; ++f){
        for(int m=0;m<k2;++m){
            out<<qR[f][m]<<"\t";
        }
    }

    out<<"\n";
out<<fixed<<setprecision(3);
    out<<"JRpti (p-val)\t"<<fixed<<setprecision(3)<<JR[0]<<fixed<<setprecision(4) <<" ("<<pJR[0]<<")"<<"\t";
    out<<"JRpsi (p-val)\t"<<fixed<<setprecision(3)<<JR[1]<<fixed<<setprecision(4)<<" ("<<pJR[1]<<")"<<"\t";
    out<<"JRpss (p-val)\t"<<fixed<<setprecision(3)<<JR[2]<<fixed<<setprecision(4)<<" ("<<pJR[2]<<")"<<"\t";
    out<<"JRps1 (p-val)\t"<<fixed<<setprecision(3)<<JR[3]<<fixed<<setprecision(4)<<" ("<<pJR[3]<<")"<<"\t";
    out<<"JRps2 (p-val)\t"<<fixed<<setprecision(3)<<JR[4]<<fixed<<setprecision(4)<<" ("<<pJR[4]<<")"<<"\t";
    out<<"\n";

    out<<"Jpti (p-val)\t"<<fixed<<setprecision(3)<<J[0]<<" ("<<fixed<<setprecision(4)<<pJ[0]<<")"<<"\t";
    out<<"Jpsi (p-val)\t"<<fixed<<setprecision(3)<<J[1]<<" ("<<fixed<<setprecision(4)<<pJ[1]<<")"<<"\t";
    out<<"Jpss (p-val)\t"<<fixed<<setprecision(3)<<J[2]<<" ("<<fixed<<setprecision(4)<<pJ[2]<<")"<<"\t";

    out.close();
}

/// Write the correlation obtained for each sample of matings under Encounter-Mating model
template<typename T=double>
void writeEMSignificances(const string& ofilename,const string& programname,const string& comments,const bool massEM,const int& numfiles,const int& n1, const int& n2,const int& samplesize,const double& SL,const double& avemmax, const double& aveR,const double& cSigRpti, const double& cSigRpsi, const double& cSigRpss, const double& cSigRps1, const double& cSigRps2, const double& cSigpti, const double& cSigpsi, const double& cSigpss, const vector<double>& aveJRpti, const vector<double>& sdJRpti, const vector<T>& aveJRpsi,const vector<double>& sdJRpsi, const vector<T>& aveJRpss,const vector<double>& sdJRpss, const T& aveJpti, const T& sdJpti,const T& aveJpsi, const T& sdJpsi, const T& aveJpss, const T& sdJpss, const vector<int>& numfilesR){

    ofstream out(ofilename.c_str(),ios::out);
    // Set the current local date
    time_t rawtime;
    struct tm * timeinfo;

    time ( &rawtime );
    timeinfo = localtime ( &rawtime );

    out<<asctime(timeinfo)<<endl;
    out<<"------------------------------------------------\n";

    string mensaje= programname +"\n";

    mensaje+=
	"---------------------------------------------------------------------\n";
	mensaje+="\n";
	mensaje+="by A Carvajal-Rodriguez			\n";
	mensaje+="http://acraaj.webs.uvigo.es				\n";
	mensaje+="---------------------------------------------------------------------\n\n";

    mensaje+=comments+"\n";
	out<<mensaje;

    string matingmodel = "\nEncounter model: ";

    if(massEM)
        matingmodel+="Mass EM\n";
    else
        matingmodel+="Indiv EM\n";

    out<<matingmodel<<endl;

    out<<"#Sampling over an average of "+NtoS(aveR)+ " mating rounds with "+NtoS(avemmax) +" matings in average per round\n";

//    out<<fixed<<setprecision(3);

    out<<"Premating female numbers: ";
    out<<n1<<"\t";

    out<<"Premating male numbers: ";
    out<<n2<<"\t";

    out<<"Sample size: ";
    out<<samplesize<<"\t";

    out<<"Number of replicates: ";
    out<<numfiles<<"\t";

    out<<"Significance level ";
    out<<SL<<endl;

    out<<"%*JRpti\t%*JRpsi\t%*JRpss\t%*JRps1\t%*JRps2"<<endl;
    out<<100*cSigRpti<<"\t"<<100*cSigRpsi<<"\t"<<100*cSigRpss<<"\t"<<100*cSigRps1<<"\t"<<100*cSigRps2<<endl;

    out<<"%*Jpti\t%*Jpsi\t%*Jpss"<<endl;
    out<<100*cSigpti<<"\t"<<100*cSigpsi<<"\t"<<100*cSigpss<<endl;

    out<<endl;

    out<<"Round\t"<<"NumFiles"<<"\t"<<"JRPTI\t"<<"sdJRPTI\t"<<"JRPSI\t"<<"sdJRPSI\t"<<"JRPSS\t"<<"sdJRPSS"<<endl;

    int Rmax= aveJRpti.size();

    for(int r=0; r<Rmax; ++r){

        if(aveJRpti[r]!= missing_value)
            out<<(r+1)<<"\t"<<numfilesR[r]<<"\t"<<aveJRpti[r]<<"\t"<<sdJRpti[r]<<"\t"<<aveJRpsi[r]<<"\t"<<sdJRpsi[r]<<"\t"<<aveJRpss[r]<<"\t"<<sdJRpss[r]<<endl;
    }

    out<<endl;
    out<<"JPTI (var)"<<"\t"<<"JPSI (var)"<<"\t"<<"JPSS (var)"<<endl;
    out<<aveJpti<<" ("<<sdJpti<<")\t"<<aveJpsi<<" ("<<sdJpsi<<")\t"<<aveJpss<<" ("<<sdJpss<<")"<<endl;
    out.close();
}

// Compute variance of a STL container using NRC two-step algorithm p. 618
template <typename T, typename Tcol>
inline void avevar(const Tcol& data,T& ave, T& var, const T& missingval){
	auto n=data.size();

	T ep=0.0,s=0.0,p=0.0;

    for(const auto& element:data)
        if(element!=missingval){
            s+= element;
        }
        else
            --n;
    if(!n){
        ave = missingval;
        return;
    }
    ave=s/n;
	var=0.0;

	for(const auto& element:data){

		s=element-ave;
		ep+= s;
		var+= (p=s*s);
	}

	var=(var-ep*ep/n)/(n);
	if(var<0.0){var=0.0;}

}

// Compute standard deviation of a STL container using NRC two-step algorithm p. 618
template <typename T, typename Tcol>
inline void avesd(const Tcol& data,T& ave, T& sd, const T& missingval){
	auto n=data.size();

	T ep=0.0,s=0.0,p=0.0;

    for(const auto& element:data)
        if(element!=missingval){
            s+= element;
        }
        else
            --n;
    if(!n){
        ave = missingval;
        return;
    }
    ave=s/n;
	sd=0.0;

	for(const auto& element:data){

		s=element-ave;
		ep+= s;
		sd+= (p=s*s);
	}

	sd=(sd-ep*ep/n)/(n);
	if(sd<0.0){sd=0.0;}
	else sd =sqrt(sd);

}

#endif



